package jdk.management.resource.internal.inst;

import jdk.management.resource.ResourceRequest;
import jdk.management.resource.ResourceRequestDeniedException;
import jdk.management.resource.internal.ApproverGroup;
import jdk.management.resource.internal.ResourceIdImpl;
import java.nio.channels.AsynchronousSocketChannel;
import jdk.internal.instrumentation.InstrumentationMethod;
import java.io.IOException;
import java.net.SocketAddress;
import jdk.internal.instrumentation.InstrumentationTarget;

@InstrumentationTarget("sun.nio.ch.AsynchronousSocketChannelImpl")
public class AsynchronousSocketChannelImplRMHooks
{
    public AsynchronousSocketChannelImplRMHooks() {
    }
    
    @InstrumentationMethod
    public final SocketAddress getLocalAddress() throws IOException {
        return this.getLocalAddress();
    }
    
    @InstrumentationMethod
    public final AsynchronousSocketChannel bind(SocketAddress socketAddress) throws IOException {
        ResourceIdImpl of = null;
        ResourceRequest approver = null;
        long request = 0L;
        if (this.getLocalAddress() == null) {
            of = ResourceIdImpl.of(socketAddress);
            approver = ApproverGroup.SOCKET_OPEN_GROUP.getApprover(this);
            try {
                request = approver.request(1L, of);
                if (request < 1L) {
                    throw new IOException("Resource limited: too many open socket channels");
                }
            }
            catch (ResourceRequestDeniedException cause) {
                throw new IOException("Resource limited: too many open socket channels", cause);
            }
        }
        int n = 0;
        AsynchronousSocketChannel bind = null;
        try {
            bind = this.bind(socketAddress);
            n = 1;
        }
        finally {
            if (approver != null) {
                approver.request(-(request - (long)n), of);
            }
        }
        return bind;
    }
}
